"""Utilities for interacting with Poetry projects."""

from __future__ import annotations

from pathlib import Path
import shlex
import subprocess
import tempfile
import tomllib
import requests as request


class Poetry:
    def __init__(self, working_dir: Path | None = None):
        self.working_dir = working_dir

    @classmethod
    def init_project(cls, path: Path | None = None) -> "Poetry":
        """Initialise a new Poetry project in ``path``.

        If ``path`` is ``None`` a temporary directory will be created.  The
        returned :class:`Poetry` instance has its ``working_dir`` set to the
        project directory.
        """

        if path is None:
            temp_dir = tempfile.TemporaryDirectory()
            project_path = Path(temp_dir.name)
            project_path._temp_dir = temp_dir  # type: ignore[attr-defined]
        else:
            project_path = Path(path)
            project_path.mkdir(parents=True, exist_ok=True)

        subprocess.run(
            shlex.split("poetry init -n"),
            check=True,
            capture_output=True,
            text=True,
            cwd=project_path,
        )

        return cls(working_dir=project_path)

    def is_poetry_project(self, directory: Path | None = None) -> bool:
        """Return ``True`` if ``directory`` contains a Poetry project."""

        dir_path = Path(directory) if directory is not None else self.working_dir
        if dir_path is None:
            return False

        pyproject = dir_path / "pyproject.toml"
        if not pyproject.is_file():
            return False

        try:
            data = tomllib.loads(pyproject.read_text(encoding="utf-8"))
        except (OSError, tomllib.TOMLDecodeError):
            return False

        tool = data.get("tool", {})
        if "poetry" in tool:
            return True

        project = data.get("project")
        build_backend = data.get("build-system", {}).get("build-backend")
        if project and build_backend == "poetry.core.masonry.api":
            return True

        return False

    def version_bump_minor(self) -> None:
        subprocess.run(
            shlex.split("poetry version minor"),
            check=True,
            capture_output=True,
            text=True,
            cwd=self.working_dir,
        )

    def version_read(self) -> str:
        output = subprocess.run(
            shlex.split("poetry version -s"),
            check=True,
            capture_output=True,
            text=True,
            cwd=self.working_dir,
        )
        return output.stdout.strip()

    def is_published(self, package_name: str) -> bool:
        url = f"https://pypi.org/pypi/{package_name}/json"
        response = request.get(url)
        return response.status_code != 404

    def is_current_project_published(self) -> bool:
        project_name = self.project_name()
        return self.is_published(project_name)

    def publish(self) -> bool:
        try:
            subprocess.run(
                shlex.split("poetry publish --build"),
                check=True,
                capture_output=True,
                text=True,
                cwd=self.working_dir,
            )
        except subprocess.CalledProcessError as e:
            print("STDOUT:\n", e.stdout)
            print("STDERR:\n", e.stderr)
            raise
        return True

    def project_name(self) -> str:
        output = subprocess.run(
            shlex.split("poetry version"),
            check=True,
            capture_output=True,
            text=True,
            cwd=self.working_dir,
        )
        return output.stdout.split()[0].strip()


__all__ = ["Poetry"]




from pathlib import Path
from typing import Optional

import semver
import typer
from typer import Typer

from vrz.core import VersionSubstitution, Vrz
from vrz.git_utils import Git
from vrz.poetry_utils import Poetry


def _bump_minor(version: str) -> str:
    """Return a version string with the minor part incremented."""
    return str(semver.VersionInfo.parse(version).bump_minor())

def main():
    poetry = Poetry()
    git = Git()
    version_substitution = VersionSubstitution()
    vrz = Vrz(Path.cwd())
    
    app = Typer(
        no_args_is_help=True, 
        help="vrz simplifies versioning and releases of software packages. Created primarily for Python, but can be used with other language platforms as well."
        )

    @app.command()
    def minor(update_file: Optional[list[str]] = typer.Option(default=None)):
        is_poetry_project = poetry.is_poetry_project(Path.cwd())

        if not git.is_git_repo():
            typer.echo("Not a git repository.")
            raise typer.Exit(code=1)

        if is_poetry_project:
            old_version = poetry.version_read()
            poetry.version_bump_minor()
            new_version = poetry.version_read()
            typer.echo(f"Version bumped to {new_version}.")
        else:
            tags = git.list_tags()
            old_version = tags[-1].lstrip("v") if tags else "0.0.0"
            new_version = _bump_minor(old_version)
            typer.echo(f"Version bumped to {new_version}.")

        tag_name = f"v{new_version}"

        if is_poetry_project:
            git.add("pyproject.toml")
            git.commit(f"Released {tag_name}.")
            git.push()
            typer.echo("Pushed updated pyproject.toml.")

        git.create_tag(tag_name)
        git.push_tag(tag_name)
        typer.echo(f"Git tag {tag_name} created and pushed.")

        if update_file:
            for file in update_file:
                typer.echo(f"Updating version in file: {file}")
                version_substitution.replace_version(file, old_version, new_version)
                git.add(file)
            git.commit(f"Updated version to {new_version}.")
            git.push()
            typer.echo("Pushed updated files.")

        if is_poetry_project and poetry.is_current_project_published():
            typer.echo("Publishing package to PyPI.")
            poetry.publish()
            typer.echo("Publishing to PyPI done.")

    @app.command()
    def latest():
        """Get the latest version of the project."""
        typer.echo(vrz.latest())

    app()

if __name__ == "__main__":
    main()



"""Core library for vrz.
"""

from pathlib import Path

from semver import Version
from vrz.git_utils import Git
from vrz.poetry_utils import Poetry


class Vrz:
    def __init__(self, path: Path = Path.cwd()):
        self.path = path
        self.poetry = Poetry(path)
        self.git = Git(path)
        self.version_substitution = VersionSubstitution()

    def latest(self) -> Version:
        if self.poetry.is_poetry_project():
            version_str = self.poetry.version_read()
            return Version.parse(version_str)
        else:
            return self.git.latest_version() or Version.parse("0.0.0")

class VersionSubstitution:
    """Replace occurrences of a version string in a file."""

    def replace_version(self, file_path: str, old_version: str, new_version: str):
        with open(file_path, "r") as file:
            content = file.read()

        new_content = content.replace(old_version, new_version)

        with open(file_path, "w") as file:
            file.write(new_content)


__all__ = ["VersionSubstitution"]




"""Utilities for interacting with Git."""

from __future__ import annotations

from pathlib import Path
import shlex
import subprocess
from typing import Optional

from semver import Version


class Git:
    """Simple wrapper around common Git commands."""

    def __init__(self, path: Path = Path.cwd()) -> None:
        self.path = path

    def is_git_repo(self) -> bool:
        try:
            subprocess.run(
                shlex.split("git rev-parse --is-inside-work-tree"),
                check=True,
                capture_output=True,
                text=True,
                cwd=self.path,
            )
            return True
        except subprocess.CalledProcessError:
            return False

    def fetch_tags(self) -> None:
        subprocess.run(
            shlex.split("git fetch --tags"),
            check=True,
            capture_output=True,
            text=True,
            cwd=self.path,
        )

    def create_tag(self, tag_name: str) -> None:
        subprocess.run(
            shlex.split(f"git tag {tag_name}"),
            check=True,
            capture_output=True,
            text=True,
            cwd=self.path,
        )

    def push_tag(self, tag_name: str) -> None:
        subprocess.run(
            shlex.split(f"git push origin {tag_name}"),
            check=True,
            capture_output=True,
            text=True,
            cwd=self.path,
        )

    def push(self) -> None:
        subprocess.run(
            shlex.split("git push"),
            check=True,
            capture_output=True,
            text=True,
            cwd=self.path,
        )

    def add(self, file: str) -> None:
        subprocess.run(
            shlex.split(f"git add {file}"),
            check=True,
            capture_output=True,
            text=True,
            cwd=self.path,
        )

    def commit(self, message: str) -> None:
        subprocess.run(
            shlex.split(f"git commit -m '{message}'"),
            check=True,
            capture_output=True,
            text=True,
            cwd=self.path,
        )

    def list_tags(self) -> list[str]:
        """Return list of Git tags sorted alphanumerically ascending."""

        result = subprocess.run(
            shlex.split("git tag --list --sort=version:refname"),
            check=True,
            capture_output=True,
            text=True,
            cwd=self.path,
        )
        tags = result.stdout.strip().splitlines()
        return tags

    def list_version_tags(self) -> list[str]:
        """Return list of Git version tags sorted alphanumerically ascending."""
        return [tag for tag in self.list_tags() if tag.startswith("v")]
    
    def latest_version_tag(self) -> Optional[str]:
        version_tags = self.list_version_tags()
        return version_tags[-1] if version_tags else None
    
    def latest_version(self) -> Optional[Version]:
        latest_tag = self.latest_version_tag()
        if latest_tag:
            return Version.parse(latest_tag.lstrip("v"))
        else:
            return None